# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.messaging.v1.brand_registration import BrandRegistrationList
from twilio.rest.messaging.v1.campaign import CampaignList
from twilio.rest.messaging.v1.deactivation import DeactivationsList
from twilio.rest.messaging.v1.service import ServiceList
from twilio.rest.messaging.v1.use_case import UseCaseList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Messaging

        :returns: V1 version of Messaging
        :rtype: twilio.rest.messaging.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._brand_registrations = None
        self._campaigns = None
        self._deactivations = None
        self._services = None
        self._use_cases = None

    @property
    def brand_registrations(self):
        """
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationList
        """
        if self._brand_registrations is None:
            self._brand_registrations = BrandRegistrationList(self)
        return self._brand_registrations

    @property
    def campaigns(self):
        """
        :rtype: twilio.rest.messaging.v1.campaign.CampaignList
        """
        if self._campaigns is None:
            self._campaigns = CampaignList(self)
        return self._campaigns

    @property
    def deactivations(self):
        """
        :rtype: twilio.rest.messaging.v1.deactivation.DeactivationsList
        """
        if self._deactivations is None:
            self._deactivations = DeactivationsList(self)
        return self._deactivations

    @property
    def services(self):
        """
        :rtype: twilio.rest.messaging.v1.service.ServiceList
        """
        if self._services is None:
            self._services = ServiceList(self)
        return self._services

    @property
    def use_cases(self):
        """
        :rtype: twilio.rest.messaging.v1.use_case.UseCaseList
        """
        if self._use_cases is None:
            self._use_cases = UseCaseList(self)
        return self._use_cases

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1>'
